Subject: [bug#54918] [WIP] make guix-service-type more extendable

From: Justin Veilleux <[email-masked]>
Date: Sun, 1 May 2022 11:35:40 -0400

Hi, I updated the documentation (to the best of my abilities, I've never 
written texinfo).

For the changelog, I'm not familiar with the format for bigger changes, 
could you point me to the relevant documentation?

Cheers
From 314f6f1c8cb1057e6880f1776d096a615c2c5a30 Mon Sep 17 00:00:00 2001
From: terramorpha <[email-masked]>
Date: Mon, 11 Apr 2022 00:30:07 -0400
Subject: [PATCH] commit message

---
 doc/guix.texi         | 21 ++++++++++++++++++
 gnu/services/base.scm | 50 ++++++++++++++++++++++++++++++++++---------
 2 files changed, 61 insertions(+), 10 deletions(-)

diff --git a/doc/guix.texi b/doc/guix.texi
index 54793c9648..910183a8d2 100644
--- a/doc/guix.texi
+++ b/doc/guix.texi
@@ -102,6 +102,7 @@ Copyright @copyright{} 2021 Sarah Morgensen@*
 Copyright @copyright{} 2021 Josselin Poiret@*
 Copyright @copyright{} 2022 Remco van 't Veer@*
 Copyright @copyright{} 2022 Aleksandr Vityazev@*
+Copyright @copyright{} 2022 Justin Veilleux@*
 
 Permission is granted to copy, distribute and/or modify this document
 under the terms of the GNU Free Documentation License, Version 1.3 or
@@ -17147,6 +17148,26 @@ A directory path where the @command{guix-daemon} will perform builds.
 @end table
 @end deftp
 
+@anchor{guix-extension-type}
+@deftp {Data Type} guix-extension
+
+This data type represents the parameters of the Guix build daemon that
+are extendable. This is the type of the object that must be used within
+a guix service extension.
+@xref{Service Composition}, for more information.
+
+@table @asis
+@item @code{authorized-keys} (default: @code{'()})
+A list of file-like objects where each element contains a public key.
+
+@item @code{substitute-urls} (default: @code{'()})
+A list of strings where each element is a substitute URL.
+
+@item @code{chroot-directories} (default: @code{'()})
+A list of file-like objects or strings pointing to additional directories the build daemon can use.
+@end table
+@end deftp
+
 @deffn {Scheme Procedure} udev-service [#:udev @var{eudev} #:rules @code{'()}]
 Run @var{udev}, which populates the @file{/dev} directory dynamically.
 udev rules can be provided as a list of files through the @var{rules}
diff --git a/gnu/services/base.scm b/gnu/services/base.scm
index 5d7c69a9cd..fc799cb806 100644
--- a/gnu/services/base.scm
+++ b/gnu/services/base.scm
@@ -17,6 +17,7 @@
 ;;; Copyright © 2021 Hui Lu <[email-masked]>
 ;;; Copyright © 2021, 2022 Maxim Cournoyer <[email-masked]>
 ;;; Copyright © 2022 Guillaume Le Vaillant <[email-masked]>
+;;; Copyright © 2022 Justin Veilleux <[email-masked]>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -187,6 +188,12 @@ (define-module (gnu services base)
             guix-configuration-extra-options
             guix-configuration-log-file
 
+            guix-extension
+            guix-extension?
+            guix-extension-authorized-keys
+            guix-extension-substitute-urls
+            guix-extension-chroot-directories
+
             guix-service-type
             guix-publish-configuration
             guix-publish-configuration?
@@ -1788,6 +1795,25 @@ (define* (references-file item #:optional (name "references"))
                                  #:references-graphs (("graph" ,item))))
       (plain-file name "()")))
 
+(define-record-type* <guix-extension>
+  guix-extension make-guix-extension
+  guix-extension?
+  (authorized-keys guix-extension-authorized-keys ;list of file-like
+                    (default '()))
+  (substitute-urls guix-extension-substitute-urls ;list of strings
+                    (default '()))
+  (chroot-directories guix-extension-chroot-directories ;list of file-like/strings
+                      (default '())))
+
+(define (guix-extension-merge a b)
+  (guix-extension
+   (authorized-keys (append (guix-extension-authorized-keys a)
+                            (guix-extension-authorized-keys b)))
+   (substitute-urls (append (guix-extension-substitute-urls a)
+                            (guix-extension-substitute-urls b)))
+   (chroot-directories (append (guix-extension-chroot-directories a)
+                               (guix-extension-chroot-directories b)))))
+
 (define guix-service-type
   (service-type
    (name 'guix)
@@ -1798,30 +1824,34 @@ (define guix-service-type
           (service-extension profile-service-type
                              (compose list guix-configuration-guix))))
 
-   ;; Extensions can specify extra directories to add to the build chroot.
-   (compose concatenate)
-   (extend (lambda (config directories)
+   ;; Extensions can specify extra directories to add to the build chroot,
+   ;; extra substitute urls and extra authorized keys
+   (compose (lambda (args) (fold guix-extension-merge (guix-extension) args)))
+   (extend (lambda (config extension)
              (guix-configuration
               (inherit config)
+              (authorized-keys (append (guix-extension-authorized-keys extension)
+                                       (guix-configuration-authorized-keys config)))
+              (substitute-urls (append (guix-extension-substitute-urls extension)
+                                       (guix-configuration-substitute-urls config)))
               (chroot-directories
-               (append (guix-configuration-chroot-directories config)
-                       directories)))))
+               (append (guix-extension-chroot-directories extension)
+                       (guix-configuration-chroot-directories config))))))
 
    (default-value (guix-configuration))
    (description
     "Run the build daemon of GNU@tie{}Guix, aka. @command{guix-daemon}.")))
 
-
 (define-record-type* <guix-publish-configuration>
   guix-publish-configuration make-guix-publish-configuration
   guix-publish-configuration?
-  (guix    guix-publish-configuration-guix        ;file-like
+  (guix    guix-publish-configuration-guix ;file-like
            (default guix))
-  (port    guix-publish-configuration-port        ;number
+  (port    guix-publish-configuration-port ;number
            (default 80))
-  (host    guix-publish-configuration-host        ;string
+  (host    guix-publish-configuration-host ;string
            (default "localhost"))
-  (advertise? guix-publish-advertise?       ;boolean
+  (advertise? guix-publish-advertise?   ;boolean
               (default #f))
   (compression       guix-publish-configuration-compression
                      (thunked)
-- 
2.34.0